package com.agentfactory.agentspeak.compiler;

import java.util.LinkedList;
import java.util.List;

import com.agentfactory.agentspeak.interpreter.Utilities;
import com.agentfactory.agentspeak.interpreter.mentalstate.AssignmentStep;
import com.agentfactory.agentspeak.interpreter.mentalstate.BasicStep;
import com.agentfactory.agentspeak.interpreter.mentalstate.DoStep;
import com.agentfactory.agentspeak.interpreter.mentalstate.ForEachStep;
import com.agentfactory.agentspeak.interpreter.mentalstate.IPlanStep;
import com.agentfactory.agentspeak.interpreter.mentalstate.IfStep;
import com.agentfactory.agentspeak.interpreter.mentalstate.TryRecoverStep;
import com.agentfactory.agentspeak.interpreter.mentalstate.WhileStep;
import com.agentfactory.logic.update.lang.AND;
import com.agentfactory.logic.update.lang.Predicate;

public class PlanBuilderVisitor implements AgentSpeakParserVisitor {

	@SuppressWarnings("unchecked")
	@Override
	public Object visit(SimpleNode node, Object data) {
		if (node.toString().equals("Plan")) {
			
			List<IPlanStep> plan = new LinkedList<IPlanStep>();
			for (int i=0; i<node.jjtGetNumChildren(); i++) {
				String fos = "";
				SimpleNode stepNode = (SimpleNode) node.jjtGetChild(i);
				if (stepNode.toString().equals("Step")) {
					if (stepNode.jjtGetValue() != null) {
						fos += stepNode.jjtGetValue();
					}
					fos += new PredicateBuilderVisitor().visit((SimpleNode) stepNode.jjtGetChild(0), null);
					plan.add(new BasicStep(Utilities.factory.createFormula(fos)));
				}
				if (stepNode.toString().equals("Statement")) {
					if (stepNode.jjtGetChild(0).toString().equals("While")) {
						SimpleNode whileNode = (SimpleNode) stepNode.jjtGetChild(0); 
						AND and = (AND) new SentenceBuilderVisitor().visit((SimpleNode) whileNode.jjtGetChild(0), null);
						List<IPlanStep> subplan = (List<IPlanStep>) new PlanBuilderVisitor().visit((SimpleNode) whileNode.jjtGetChild(1), null);
						plan.add(new WhileStep(and, subplan));
					} else if (stepNode.jjtGetChild(0).toString().equals("TryRecover")) {
							SimpleNode tryNode = (SimpleNode) stepNode.jjtGetChild(0); 
							List<IPlanStep> tryPlan = (List<IPlanStep>) new PlanBuilderVisitor().visit((SimpleNode) tryNode.jjtGetChild(0), null);
							List<IPlanStep> recoverPlan = (List<IPlanStep>) new PlanBuilderVisitor().visit((SimpleNode) tryNode.jjtGetChild(1), null);
							plan.add(new TryRecoverStep(tryPlan, recoverPlan));
					} else if (stepNode.jjtGetChild(0).toString().equals("If")) {
						SimpleNode whileNode = (SimpleNode) stepNode.jjtGetChild(0); 
						AND and = (AND) new SentenceBuilderVisitor().visit((SimpleNode) whileNode.jjtGetChild(0), null);
						List<IPlanStep> ifPlan = (List<IPlanStep>) new PlanBuilderVisitor().visit((SimpleNode) whileNode.jjtGetChild(1), null);
						List<IPlanStep> elsePlan;
						if (whileNode.jjtGetNumChildren() == 3) {
							elsePlan = (List<IPlanStep>) new PlanBuilderVisitor().visit((SimpleNode) whileNode.jjtGetChild(2), null);
						} else {
							elsePlan = new LinkedList<IPlanStep>();
						}
						plan.add(new IfStep(and, ifPlan, elsePlan));
					} else if (stepNode.jjtGetChild(0).toString().equals("ForEach")) {
						SimpleNode whileNode = (SimpleNode) stepNode.jjtGetChild(0); 
						AND and = (AND) new SentenceBuilderVisitor().visit((SimpleNode) whileNode.jjtGetChild(0), null);
						List<IPlanStep> subplan = (List<IPlanStep>) new PlanBuilderVisitor().visit((SimpleNode) whileNode.jjtGetChild(1), null);
						plan.add(new ForEachStep(and, subplan));
					} else if (stepNode.jjtGetChild(0).toString().equals("Assignment")) {
						SimpleNode assignNode = (SimpleNode) stepNode.jjtGetChild(0);
						String variable = (String) assignNode.jjtGetValue();
						
						String term = (String) new PredicateBuilderVisitor().visit((SimpleNode) assignNode.jjtGetChild(0), null);
						plan.add(new AssignmentStep(Utilities.factory.createTerm(variable), Utilities.factory.createTerm(term)));
					} else if (stepNode.jjtGetChild(0).toString().equals("Do")) {
						SimpleNode doNode = (SimpleNode) stepNode.jjtGetChild(0);
						String module = (String) new PredicateBuilderVisitor().visit((SimpleNode) doNode.jjtGetChild(0), null);
						String action = (String) new PredicateBuilderVisitor().visit((SimpleNode) doNode.jjtGetChild(1), null);
						plan.add(new DoStep(Utilities.factory.createTerm(module), (Predicate) Utilities.factory.createFormula(action)));
					}
				}
			}
			return plan;
		}
		return null;
	}

}
