package com.agentfactory.agentspeak.core;

import com.agentfactory.agentspeak.interpreter.Utilities;
import com.agentfactory.logic.update.interpreter.Action;
import com.agentfactory.logic.update.interpreter.Module;
import com.agentfactory.logic.update.lang.Predicate;
import com.agentfactory.platform.core.AgentCreationException;
import com.agentfactory.platform.core.DuplicateAgentNameException;
import com.agentfactory.platform.core.NoSuchArchitectureException;
import com.agentfactory.service.ams.AgentManagementService;

public class AMSModule extends Module {
	AgentManagementService service;
	
	public AMSModule() {
		addAction("setup", new Action() {
			@Override
			public boolean execute(Predicate activity) {
				agent.bindToPlatformService(AgentManagementService.NAME);
				service = (AgentManagementService) agent.getPlatformService(AgentManagementService.NAME);
				return true;
			}
		});
		addAction("createAgent(?name, ?design)", new Action() {
			@Override
			public boolean execute(Predicate activity) {
				String name = Utilities.presenter.toString(activity.termAt(0));
				String design = Utilities.presenter.toString(activity.termAt(1));
				try {
					service.createAgent(name, design);
				} catch (NoSuchArchitectureException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
					return false;
				} catch (DuplicateAgentNameException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
					return false;
				} catch (AgentCreationException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
					return false;
				}
				return true;
			}
		});
		addAction("terminateAgent(?name)", new Action() {
			@Override
			public boolean execute(Predicate activity) {
				String name = Utilities.presenter.toString(activity.termAt(0));
				service.terminateAgent(name);
				return true;
			}
		});
		addAction("suspendAgent(?name)", new Action() {
			@Override
			public boolean execute(Predicate activity) {
				String name = Utilities.presenter.toString(activity.termAt(0));
				service.suspendAgent(name);
				return true;
			}
		});
		addAction("resumeAgent(?name)", new Action() {
			@Override
			public boolean execute(Predicate activity) {
				String name = Utilities.presenter.toString(activity.termAt(0));
				service.resumeAgent(name);
				return true;
			}
		});
	}
}
