package com.agentfactory.agentspeak.interpreter;

import com.agentfactory.agentspeak.compiler.AgentSpeakParser;
import com.agentfactory.agentspeak.compiler.CodeGeneratorVisitor;
import com.agentfactory.agentspeak.compiler.IncludesExpanderStream;
import com.agentfactory.agentspeak.compiler.SimpleNode;
import com.agentfactory.agentspeak.interpreter.mentalstate.PlanRule;
import com.agentfactory.logic.update.interpreter.Action;
import com.agentfactory.logic.update.interpreter.Module;
import com.agentfactory.logic.update.interpreter.Sensor;
import com.agentfactory.logic.update.lang.Predicate;
import com.agentfactory.platform.core.AgentCreationException;

public class ScriptBasedAgentSpeakAgent extends AbstractAgentSpeakAgent {

	public ScriptBasedAgentSpeakAgent(String name) {
		super(name);
	}

	public void construct(String design) throws AgentCreationException {
		CodeGeneratorVisitor cgv = new CodeGeneratorVisitor();
		try {
			AgentSpeakParser parser = new AgentSpeakParser(new IncludesExpanderStream(design));
			SimpleNode n = parser.Start();
			cgv.visit(n, null);
			if (!parser.errors.isEmpty()) {
				throw new RuntimeException("ERROR: AgentSpeak program does not compile: " + design);
			}
			
			for (Predicate belief: cgv.beliefs) {
				beliefs.addBelief(belief);
			}
			
			for (Predicate id: cgv.actions.keySet()) {
				Action action = (Action) Class.forName(cgv.actions.get(id)).newInstance();
				action.setId(id);
				action.setAgent(this);
				actionMap.put(Utilities.createSignature(id), action);
			}
			
			for (Predicate id: cgv.sensors.keySet()) {
				Sensor sensor = (Sensor) Class.forName(cgv.sensors.get(id)).newInstance();				
				sensor.setId(id);
				sensor.setAgent(this);
				sensorMap.put(Utilities.createSignature(id), sensor);
			}
			
			for (Predicate id: cgv.modules.keySet()) {
				addModule(id, (Module) Class.forName(cgv.modules.get(id)).newInstance());
			}
			
			for (PlanRule rule: cgv.rules) {
				addPlanRule(rule);
			}
		} catch (Throwable e) {
			e.printStackTrace();
			throw new AgentCreationException(e.getMessage());
		}
	}
}
