package com.agentfactory.agentspeak.interpreter.mentalstate;

import com.agentfactory.agentspeak.interpreter.Utilities;
import com.agentfactory.logic.update.lang.Predicate;
import com.agentfactory.logic.update.lang.Unifier;
import com.agentfactory.logic.update.reasoner.Bindings;

public class ASEvent {
	public static final int ADDITION 						= 0;
	public static final int REMOVAL							= 1;
	
	private int type;
	private Predicate content;
	private Object source;
	
	public ASEvent(int type, Predicate content) {
		this(type, content, null);
	}
	
	public ASEvent(int type, Predicate content, Object source) {
		this.type = type;
		this.content = content;
		this.source = source;
	}
	
	public Predicate content() {
		return content;
	}
	
	public boolean matches(ASEvent event) {
		return (type == event.type) && (content.matches(event.content));
	}
	
	public boolean isGoalEvent() {
		// Check if the content is an achievement goal...
		return content.functor().charAt(0) == '@';
	}
	
	public Object getSource() {
		return source;
	}
	
	public String toString() {
		String output = "+";
		if (type == REMOVAL) output = "-";
		return output + Utilities.presenter.toString(content);
	}
	
	public ASEvent apply(Bindings set) {
		return new ASEvent(type, (Predicate) content.apply(set));
	}
	
	public Bindings unifyWith(ASEvent event) {
		return Unifier.unifyFormulae(content, event.content);
	}
}
