package com.agentfactory.agentspeak.interpreter.mentalstate;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import com.agentfactory.agentspeak.interpreter.Utilities;
import com.agentfactory.logic.update.lang.IFormula;
import com.agentfactory.logic.update.lang.Predicate;
import com.agentfactory.logic.update.reasoner.IQueryable;

public class BeliefBase implements IQueryable {
	private List<Predicate> beliefs;
	private Map<String, List<Predicate>> beliefMap;
	
	public BeliefBase() {
		beliefs = new LinkedList<Predicate>();
		beliefMap = new HashMap<String, List<Predicate>>();
	}
	
	public boolean addBelief(Predicate belief) {
		List<Predicate> list = beliefMap.get(belief.functor());
		if (list == null) {
			list = new LinkedList<Predicate>();
			beliefMap.put(belief.functor(), list);
		} else {
			for (Predicate bel: list) {
				if (bel.equals(belief)) {
					boolean newBelief = false;
					for (Predicate label: belief.getLabels()) {
						if (!bel.getLabels().contains(label)) {
							bel.getLabels().add(label);
							newBelief = true;
						}
					}
					return newBelief;
				}
			}
		}
		list.add(belief);
		beliefs.add(belief);
		return true;
	}
	
	public boolean removeBelief(Predicate belief) {
		List<Predicate> list = beliefMap.get(belief.functor());
		if (list != null) {
			int i = list.indexOf(belief);
			if (i == -1) {
				return false;
			}

			if (belief.getLabels().isEmpty()) {
				list.remove(i);
				beliefs.remove(belief);
				return true;
			}
			// remove any listed sources from the belief and check
			// if there are any remaining sources. If there are,
			// keep the belief
			Predicate bel = list.get(i);
			boolean modified = false;
			for (Predicate label: belief.getLabels()) {
				if (bel.getLabels().remove(label)) {
					modified = true;
				}
			}
			
			if (bel.getLabels().isEmpty()) {
				if (list.remove(belief)) {
					beliefs.remove(belief);
				}
			}
			
			return modified;
		}
		return false;
	}

	@Override
	public boolean hasType(String type) {
		return true;
	}
	
	public List<Predicate> getBeliefs() {
		return beliefs;
	}

	@Override
	public List<IFormula> query(IFormula query) {
		List<IFormula> list = new LinkedList<IFormula>();
		if (Predicate.class.isInstance(query)) {
			Predicate predicate = (Predicate) query;
			List<Predicate> bels = beliefMap.get(predicate.functor());
			if (bels != null) {
				for (Predicate fos: bels) {
					if (fos.matches(predicate)) {
						list.add(fos);
					}
				}
			}
		}
		return list;
	}
	
	public void wipe(String label) {
		Predicate lab = (Predicate) Utilities.factory.createFormula("source("+label+")");
		int index = 0;
		while (index < beliefs.size()) {
			if (Predicate.class.isInstance(beliefs.get(index))) {
				Predicate predicate = (Predicate) beliefs.get(index);
				int i = 0;
				while (i < predicate.getLabels().size()) {
					if (predicate.getLabels().get(i).equals(lab)) {
						predicate.getLabels().remove(i);
					} else {
						i++;
					}
				}
				if (predicate.getLabels().isEmpty()) {
					 beliefs.remove(index);
					 List<Predicate> list = beliefMap.get(predicate.functor());
					 list.remove(predicate);
				} else {
					index++;
				}
			}
		}
	}
}
