package com.agentfactory.agentspeak.interpreter.mentalstate;

import java.util.LinkedList;
import java.util.List;

import com.agentfactory.agentspeak.interpreter.Utilities;
import com.agentfactory.logic.update.lang.IFormula;
import com.agentfactory.logic.update.lang.Predicate;
import com.agentfactory.logic.update.reasoner.Bindings;

public class PlanRule {
	private ASEvent triggeringEvent;
	private IFormula context;
	private List<IPlanStep> plan;
	private Predicate label;
	
	public PlanRule(Predicate label, ASEvent event, IFormula context, List<IPlanStep> plan) {
		this.label = label;
		this.triggeringEvent = event;
		this.context = context;
		this.plan = plan;
	}
	
	public Predicate getLabel() {
		return label;
	}
	
	public ASEvent getTriggeringEvent() {
		return triggeringEvent;
	}
	
	public IFormula getContext() {
		return context;
	}
	
	public List<IPlanStep> getPlan() {
		return plan;
	}
	
	public boolean isOption(ASEvent event) {
		return triggeringEvent.matches(event);
	}
	
	public String toString() {
		String output = "@" + Utilities.presenter.toString(label) + "\n" +
						triggeringEvent.toString() + " : " +
						Utilities.presenter.toString(context) + " <-";
		boolean first = true;
		for (IPlanStep step: plan) {
			if (first) first = false;
			else output += ",";
			output += "\n\t" + step.toString();
		}
		return output + ";";
	}
	
	public PlanRule apply(Bindings set) {
		List<IPlanStep> list = new LinkedList<IPlanStep>();
		for (IPlanStep step: plan) {
			list.add(step.apply(set));
		}
		return new PlanRule((Predicate) label.apply(set), triggeringEvent.apply(set), context.apply(set), list);
	}
}
