package com.agentfactory.agentspeak.interpreter.mentalstate;

import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import com.agentfactory.logic.update.lang.IFormula;
import com.agentfactory.logic.update.lang.Variable;
import com.agentfactory.logic.update.reasoner.Bindings;

public class StateUpdateFormula implements IFormula {
	public static final int ADDITION 						= 0;
	public static final int REMOVAL							= 1;
	
	private IFormula formula;
	private int type;
//	private Object source;
	
	public StateUpdateFormula(int type, IFormula formula) {
		this.type = type;
		this.formula = formula;
	}
	
	public int updateType() {
		return type;
	}
	
	public IFormula formula() {
		return formula;
	}

	@Override
	public IFormula apply(Bindings set) {
		return new StateUpdateFormula(type, formula.apply(set));
	}

	@Override
	public List<IFormula> asList() {
		List<IFormula> list = new LinkedList<IFormula>();
		list.add(formula);
		return list;
	}

	@Override
	public Set<Variable> getVariables() {
		return formula.getVariables();
	}

	@Override
	public boolean isFalse() {
		return false;
	}

	@Override
	public boolean isTrue() {
		return false;
	}

	@Override
	public boolean matches(IFormula formula) {
		if (StateUpdateFormula.class.isInstance(formula)) {
			return this.formula.matches(((StateUpdateFormula) formula).formula); 
		}
		return false;
	}

	@Override
	public String type() {
		return formula.type();
	}

}
