package com.agentfactory.logic.update.lang;

import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import com.agentfactory.logic.update.interpreter.CoreUtilities;
import com.agentfactory.logic.update.reasoner.Bindings;

public class Equality implements IFormula {
	private ITerm left;
	private ITerm right;
	
	public Equality(ITerm left, ITerm right) {
		this.left = left;
		this.right = right;
	}
	
	@Override
	public IFormula apply(Bindings set) {
		return new Equality(left.apply(set), right.apply(set));
	}
	
	@Override
	public List<IFormula> asList() {
		List<IFormula> list = new LinkedList<IFormula>();
		list.add(this);
		return list;
	}
	@Override
	public Set<Variable> getVariables() {
		Set<Variable> vars = new HashSet<Variable>();
		vars.addAll(left.getVariables());
		vars.addAll(right.getVariables());
		return vars;
	}
	@Override
	public boolean isFalse() {
		return !left.equals(right);
	}
	@Override
	public boolean isTrue() {
		return left.equals(right);
	}
	@Override
	public boolean matches(IFormula formula) {
		if (Equality.class.isInstance(formula)) {
			Equality e = (Equality) formula;
			return left.equals(e.left) && right.equals(e.right);
		}
		return false;
	}
	
	@Override
	public String type() {
		return "raw-logic";
	}
	
	public ITerm left() {
		return left;
	}
	
	public ITerm right() {
		return right;
	}
}
