package com.agentfactory.platform.impl;

import com.agentfactory.platform.core.AgentID;
import com.agentfactory.platform.core.IAgent;
import com.agentfactory.platform.core.IMessage;

/**
 * This abstract class outlines the methods that should be implemented by any
 * message transport service.
 *
 * @author  Rem Collier
 */
public abstract class MessageTransportService extends AbstractPlatformService {

    public abstract boolean isValidService( String address );

    public abstract boolean sendMessage( String address, IMessage message );

    public abstract String getAddress();

    public abstract String getServiceID();

    @Override
    public void onBind( IAgent agent ) {
        addObserver( agent );
    }

    @Override
    public void onUnbind( IAgent agent ) {
        deleteObserver( agent );
    }

    /**
     * This method is invoked from within a specific MTS implementation, and
     * is used to pass a received message on to the relevant agent.
     * 
     * @param message
     * @return
     */
    public synchronized boolean receiveMessage( IMessage message ) {
        AgentID aid = (AgentID) message.getReceivers().get( 0 );

        int index = aid.getName().indexOf( '*' );
        if ( index == -1 ) {
            IAgent agent = getAgentByName( aid.getName() );
            if ( agent != null ) {
                agent.receive( message );
            } else {
                return false;
            }
        } else {
            String pattern = aid.getName().substring( 0, index );

            for ( IAgent agent : getBoundAgents() ) {
                if ( agent.getName().startsWith( pattern ) ) {
                    agent.receive( message );
                }
            }
        }
        
        return true;
    }
}

