package com.agentfactory.service.mts.http;

import com.agentfactory.platform.core.Constants;
import com.agentfactory.platform.core.IEnvelope;
import com.agentfactory.platform.core.IMessage;
import com.agentfactory.platform.impl.StringMessage;
import com.agentfactory.platform.impl.XMLEnvelope;
import java.util.StringTokenizer;

/**
 * Helper class that contains various useful methods.
 *
 * @author  Rem Collier
 */
public class HTTPHelper {

    /**
     * This method unfolds a string as specified in RFC2068.  Specifically,
     * a string is folded if:
     *  - a carriage return is followed by a space.
     *  - a carriage return is followed by a tab.
     * To unfold the string, simply remove all such occurrences.
     *
     * @param source the string to be unfolded.
     * @return an unfolded version of source.
     */
    public static String unfoldString(String source) {
        StringBuffer buf = new StringBuffer();
        int index = 0;
        int index1 = source.indexOf("\n ");
        int index2 = source.indexOf("\n\t");
        while ((index1 >= 0) || (index2 >= 0)) {
            if (index1 == -1) {
                buf.append(source.substring(index, index2));
                index = index2 + 2;
            } else if (index2 == -1) {
                buf.append(source.substring(index, index1));
                index = index1 + 2;
            } else if (index1 < index2) {
                buf.append(source.substring(index, index1));
                index = index1 + 2;
            } else {
                buf.append(source.substring(index, index2));
                index = index2 + 2;
            }
            index1 = source.indexOf("\n ", index);
            index2 = source.indexOf("\n\t", index);
        }
        buf.append(source.substring(index));
        return buf.toString();
    }
    
    public static String toString(byte[] content) {
        StringBuffer buf = new StringBuffer();
        for (int i = 0; i <content.length; i++) {
            buf.append((char) content[i]);
        }
        return buf.toString();
    }
    
    public static String getFormat(String header) {
        String format = null;
        
        // Now tokenize using a carriage return and parse the header
        // information.  Really, all we are looking for here is the
        // content type, which should tell us the format that the body
        // takes!
        String name, value;
        StringTokenizer tok2 = null;
        StringTokenizer tok = new StringTokenizer(header, "\n");
        while (tok.hasMoreTokens()) {
            tok2 = new StringTokenizer(tok.nextToken(), ":");
            name = tok2.nextToken().trim();
            value = new StringTokenizer(tok2.nextToken().trim(), ";").nextToken();
            
            if (name.equals(Constants.CONTENT_TYPE)) {
                tok2 = new StringTokenizer(value, "/");
                if (tok2.nextToken().trim().equals("application")) {
                    format = tok2.nextToken().trim();
                }
            }
        }
        
        return format;
    }
    
    public static IEnvelope parseHTTPEnvelope(String text) {
        IEnvelope envelope = null;
        
        // First separate the header from the body of the envelope.
        int index = text.indexOf(Constants.CRLF);
        String header = text.substring(0, index).trim();
        String body = text.substring(index + 2).trim();
        
        String format = HTTPHelper.getFormat(header);
        if (format != null) {
            if (format.equals(Constants.FIPA_XML_ENVELOPE_TYPE)) {
                envelope = (XMLEnvelope) XMLEnvelope.newInstance(body).get(0);
            } else if (format.equals(Constants.XML_ENVELOPE_TYPE)) {
                envelope = (XMLEnvelope) XMLEnvelope.newInstance(body).get(0);
            }
        }
        
        return envelope;
    }
    

    public static IMessage parseHTTPPayload(IEnvelope envelope, String body) {
        IMessage message = null;

        if (envelope.getAclRepresentation().equals(Constants.FIPA_STRING_MESSAGE_TYPE)) {
                message = (StringMessage) StringMessage.newInstance(body);
        }
    
        return message;
    }
}
