package com.agentfactory.service.mts.local;

import com.agentfactory.platform.impl.MessageTransportService;
import com.agentfactory.platform.core.IEnvelope;
import com.agentfactory.platform.core.IMessage;
import com.agentfactory.platform.core.TransportMessage;
import com.agentfactory.platform.impl.XMLEnvelope;
import java.util.Calendar;

/**
 * Implementation of the FIPA HTTP Message Transport Service.
 *
 * @author  Rem Collier
 */
public class LocalMessageTransportService extends MessageTransportService {
    public static final String NAME = "fipa.mts.mtp.local.std";

    /**
     * Checks if a given address is valid for this service.
     *
     * @param address the address to be checked
     * @return boolean -  true if the address is valid for the service. false otherwise.
     */
    public boolean isValidService( String address ) {
        return address.startsWith( "local:" ) && address.endsWith( manager.getAgentPlatform().getFullName() );
    }

    /**
     * This method is responsible for transmitting messages using the FIPA
     * Local Message Transport Service.
     *
     * @param address A String object
     * @param message a Message object
     */
    public boolean sendMessage( String address, IMessage message ) {
        return receiveMessage( message );
    }

    public TransportMessage createTransportMessage( IMessage message ) {
        IEnvelope envelope = new XMLEnvelope();
        envelope.setTo( message.getReceivers() );
        envelope.setFrom( message.getSender() );
        envelope.setAclRepresentation( message.getAclRepresentation() );
        envelope.setDate( Calendar.getInstance() );
        envelope.setPayloadEncoding( message.getPayloadEncoding() );
        return new TransportMessage( envelope, message );
    }

    /**
     * This method creates a socket on the assigned port, and listens for
     * connection attempts.
     */
    public void onStart() {
    }

    /**
     * This method closes down the server socket, stopping any additional
     * connections being made to this message transport service.
     */
    public void onStop() {
        super.stop();
    }

    public String getAddress() {
        return "local:" + manager.getAgentPlatform().getFullName();
    }

    public String getServiceID() {
        return "local-" + manager.getAgentPlatform().getFullName();
    }
}
