package com.agentfactory.smt;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintStream;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.HashMap;
import java.util.Map;

import com.agentfactory.platform.core.IAgentPlatform;

public class CommandServer implements Runnable {
	private int port;
	private ServerSocket serverSocket;
	private IAgentPlatform platform;
	private Map<String, ICommandHandler> commands;
	private Thread thread;
	
	public CommandServer(IAgentPlatform platform, int port) {
		this(platform, port, new HashMap<String,ICommandHandler>());
	}
	
	public CommandServer(IAgentPlatform platform, int port, Map<String, ICommandHandler> commands) {
		this.port = port;
		this.platform = platform;
		this.commands = commands;
		thread = new Thread(this);
		thread.start();
	}
	
	public void addCommandHandler(ICommandHandler handler) {
		commands.put(handler.type(), handler);
	}
	
	public void defaultConfiguration() {
		addCommandHandler(new ConnectCommand(platform));
	}

	@Override
	public void run() {
		try {
			serverSocket = new ServerSocket(port);
			
			while (true) {
				Socket socket = serverSocket.accept();
				Thread thread = new Thread(new TcpThread(socket));
				thread.start();
			}
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	private class TcpThread implements Runnable {
		Socket socket;
		BufferedReader in;
		PrintStream out;
		
		public TcpThread(Socket socket) throws IOException {
			this.socket = socket;
			in = new BufferedReader(new InputStreamReader(socket.getInputStream()));
			out = new PrintStream(socket.getOutputStream());
		}
		
		@Override
		public void run() {
			try {
				String input = null;
				while ((input = in.readLine()) != null) {
					String[] command = input.split(" ");
					ICommandHandler handler = commands.get(command[0]);
					if (handler == null) {
						out.println("UNKNOWN_COMMAND");
					} else if (handler.equals("CLOSE")) {
						out.close();
						in.close();
						socket.close();
					} else {
						out.println(handler.handle(command));
					}
				}
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} 
		}
	}
	
}
