package com.agentfactory.teleoreactive.compiler;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import com.agentfactory.logic.update.lang.AND;
import com.agentfactory.logic.update.lang.IFormula;
import com.agentfactory.logic.update.lang.IMPLIES;
import com.agentfactory.logic.update.lang.ITerm;
import com.agentfactory.logic.update.lang.Predicate;
import com.agentfactory.platform.core.AgentCreationException;
import com.agentfactory.teleoreactive.interpreter.Utilities;
import com.agentfactory.teleoreactive.interpreter.mentalstate.TRProgram;
import com.agentfactory.teleoreactive.interpreter.mentalstate.TRRule;

public class CodeGeneratorVisitor implements TeleoReactiveParserVisitor {
	public Map<Predicate, String> actions = new HashMap<Predicate, String>();
	public Map<Predicate, String> sensors = new HashMap<Predicate, String>();
	public List<Predicate> beliefs = new LinkedList<Predicate>();
	public List<IMPLIES> resolutionRules = new LinkedList<IMPLIES>();
	public List<IMPLIES> backResolutionRules = new LinkedList<IMPLIES>();
	public List<IMPLIES> truthManRules = new LinkedList<IMPLIES>();
	public Map<String, TRProgram> programs = new HashMap<String, TRProgram>();
	public Map<Predicate, String> modules = new HashMap<Predicate, String>();
	private boolean hasSuper = false;
	private String superClass ="";
	public String name = "";
	private boolean isAbstract =false;
	@Override
	public Object visit(SimpleNode node, Object data) {
		PredicateConstructorVisitor visitor = new PredicateConstructorVisitor();
		if (node.toString().equals("SensorExpression")) {
			String predicate = (String) visitor.visit((SimpleNode) node.jjtGetChild(0), null);
			String cname = (String) ((SimpleNode) node.jjtGetChild(1)).jjtGetValue();
			if (cname != null) {
				sensors.put((Predicate) Utilities.factory.createFormula(predicate), cname);
			}
		} else if (node.toString().equals("ActionExpression")) {
			String predicate = (String) visitor.visit((SimpleNode) node.jjtGetChild(0), null);
			String cname = (String) ((SimpleNode) node.jjtGetChild(1)).jjtGetValue();
			if (cname != null) {
				actions.put((Predicate) Utilities.factory.createFormula(predicate), cname);
			}
		} else if (node.toString().equals("ModuleExpression")) {
			String predicate = (String) visitor.visit((SimpleNode) node.jjtGetChild(0), null);
			String cname = (String) ((SimpleNode) node.jjtGetChild(1)).jjtGetValue();
			if (cname != null) {
				modules.put((Predicate) Utilities.factory.createFormula(predicate), cname);
			}
		} else if (node.toString().equals("FuncExpression")) {
			List<TRRule> rules = new LinkedList<TRRule>();
			List<ITerm> arguments = new LinkedList<ITerm>();
			SimpleNode function = (SimpleNode) node.jjtGetChild(0);
			Predicate functiondec = (Predicate) Utilities.factory.createFormula((String) visitor.visit(function, null));
			for (int i = 0; i < functiondec.size(); i++) {
				arguments.add(functiondec.termAt(i));   
            }
			String funName = Utilities.createSignature(functiondec);
			// get name of function
			for (int i = 1; i < node.jjtGetNumChildren(); i++) {// for each rule load and add to trprog
				TRRule r = (TRRule)visit(((SimpleNode)node.jjtGetChild(i)), data);
				rules.add(r);
			}
			TRProgram prog = new TRProgram(funName, rules, arguments);
			if(node.jjtGetValue() != null){
				if(node.jjtGetValue().equals("#abstract")){
					prog.setAbstract(true);
					if(prog.getRules().size() >0)
						System.err.println("Function "+prog.getName()+" in "+name+ " is declared abstract and should not have any rules");
				} else {
					if(prog.getRules().size() == 0)
						System.err.println("Function "+prog.getName()+" in "+name+ " should be declared abstract");
				}
				if(node.jjtGetValue().equals("#override"))prog.setOverride(true);
			} else{
				if(prog.getRules().size() == 0)
					System.err.println("Function "+prog.getName()+" in "+name+ " should be declared abstract");

			}
			programs.put(funName, prog);
		} else if (node.toString().equals("ResolutioRuleExp")) {
			IMPLIES i = (IMPLIES) visit((SimpleNode) node.jjtGetChild(0), null);
			if(IMPLIES.class.isInstance(i)){
				resolutionRules.add((IMPLIES)i);
			} else{
			}
			
		} else if (node.toString().equals("BCResolutioRuleExp")) {
			IMPLIES i = (IMPLIES) visit((SimpleNode) node.jjtGetChild(0), null);
			if(IMPLIES.class.isInstance(i)){
				backResolutionRules.add((IMPLIES)i);
			}else {
				System.out.println("Shit");
			}
			
		} else if (node.toString().equals("TruthManagmentExp")) {
			IMPLIES i = (IMPLIES) visit((SimpleNode) node.jjtGetChild(0), null);
			if(IMPLIES.class.isInstance(i)){
				truthManRules.add((IMPLIES)i);
			}else {
			}
			
		} else if (node.toString().equals("Predicate")) {
			String predicate = (String) visitor.visit(node, null);
			beliefs.add((Predicate) Utilities.factory.createFormula(predicate));
		} else if(node.toString().equals("RuleExpression")){
			AND and = (AND) new SentenceBuilderVisitor().visit((SimpleNode) node.jjtGetChild(0), null);
			String predicateAction = (String) visitor.visit((SimpleNode) node.jjtGetChild(1), null);
			IFormula act = Utilities.factory.createFormula(predicateAction);
			TRRule rule = new TRRule(and, act);
			return rule;
			
		}else if (node.toString().equals("AgentExpression")) {
			name = (String) visitor.visit((SimpleNode) node.jjtGetChild(0), null);
			if(node.jjtGetNumChildren() >1){
				superClass = (String) ((SimpleNode) node.jjtGetChild(1)).jjtGetValue();
				hasSuper = true;
			}
			if(node.jjtGetValue() != null && node.jjtGetValue().equals("#abstract")){
				isAbstract = true;
			}
		} else if (node.toString().equals("ImpliesExpression")) {
			AND and = (AND) new SentenceBuilderVisitor().visit((SimpleNode) node.jjtGetChild(0), null);
			String predicateAction = (String) visitor.visit((SimpleNode) node.jjtGetChild(1), null);
			IFormula i = Utilities.factory.createFormula(Utilities.presenter.toString(and)+"=>"+predicateAction);
			return i;
		} else {
			for (int i=0; i<node.jjtGetNumChildren(); i++) {
				visit((SimpleNode) node.jjtGetChild(i), data);
			}
		}
		return null;
	}
	public boolean hasSuper() {
		return hasSuper;
	}
	public String getSuper() {
		return superClass ;
	}
	public boolean isAbstract(){
		return isAbstract ;
	}

}
