package com.agentfactory.teleoreactive.interpreter;

import java.util.LinkedList;
import java.util.List;

import com.agentfactory.logic.update.interpreter.Action;
import com.agentfactory.logic.update.interpreter.Module;
import com.agentfactory.logic.update.interpreter.Sensor;
import com.agentfactory.logic.update.lang.IFormula;
import com.agentfactory.logic.update.lang.IMPLIES;
import com.agentfactory.logic.update.lang.Predicate;
import com.agentfactory.logic.update.reasoner.QueryResult;
import com.agentfactory.platform.core.AgentCreationException;
import com.agentfactory.teleoreactive.compiler.CodeGeneratorVisitor;
import com.agentfactory.teleoreactive.compiler.IncludesExpanderStream;
import com.agentfactory.teleoreactive.compiler.SimpleNode;
import com.agentfactory.teleoreactive.compiler.TeleoReactiveParser;
import com.agentfactory.teleoreactive.interpreter.mentalstate.TRProgram;

public class ScriptBasedTeleoReactiveAgent extends AbstractTeleoReactiveAgent {

	public ScriptBasedTeleoReactiveAgent(String name) {
		super(name);
	}
	public void construct(String design) throws AgentCreationException {
		List<CodeGeneratorVisitor> agents = new LinkedList<CodeGeneratorVisitor>();
		CodeGeneratorVisitor cgv = new CodeGeneratorVisitor();
		try {
			TeleoReactiveParser parser = new TeleoReactiveParser(IncludesExpanderStream.class.getResourceAsStream("/" + design));
			SimpleNode n = parser.Start();
			cgv.visit(n, null);
			agents.add(cgv);
			
			if (!parser.errors.isEmpty()) {
				throw new RuntimeException("ERROR: Teleo-Reactive program does not compile: " + design);
			}
			while(cgv.hasSuper()){
				System.out.println("Opening agent "+cgv.getSuper().replace('.','/'));
				TeleoReactiveParser parser2 = new TeleoReactiveParser(IncludesExpanderStream.class.getResourceAsStream("/" + cgv.getSuper().replace('.','/')+".aftr"));
				cgv = new CodeGeneratorVisitor();
				n = parser2.Start();
				cgv.visit(n, null);
				agents.add(0,cgv);
			}
			
			Predicate p = (Predicate) Utilities.factory.createFormula("nil");
			cgv.actions.put(p, "com.agentfactory.teleoreactive.core.NilAction");
			
			for(CodeGeneratorVisitor c: agents){
				//System.out.println(c.name);
				getInfo(c);
			}
			checkFunctions();
			
//			beliefs.addBeliefs(cgv.beliefs);
				} catch (Exception e) {
			e.printStackTrace();
			throw new AgentCreationException(e.getMessage());
		}
	}
	private void checkFunctions() throws AgentCreationException {
		for(TRProgram t: getPrograms().values()){
			if(t.isAbstract())
				throw new AgentCreationException("Function "+t.getName()+" must be defined");
		}
		
	}
	private void getInfo(CodeGeneratorVisitor cgv) throws Exception {
		for (Predicate id: cgv.actions.keySet()) {
			Action action = (Action) Class.forName(cgv.actions.get(id)).newInstance();
			action.setId(id);
			action.setAgent(this);
			getActionMap().put(Utilities.createSignature(id), action);
		}
		
		for (Predicate id: cgv.sensors.keySet()) {
			Sensor sensor = (Sensor) Class.forName(cgv.sensors.get(id)).newInstance();				
			sensor.setId(id);
			sensor.setAgent(this);
			getSensorMap().put(Utilities.createSignature(id), sensor);
		}
		for (Predicate id: cgv.modules.keySet()) {
			Module module = (Module) Class.forName(cgv.modules.get(id)).newInstance();				
			module.setId(id);
			module.setAgent(this);
			moduleMap.put(Utilities.createSignature(id), module);
		}
		
		
		for (String key: cgv.programs.keySet()) {
			if(getPrograms().get(key) != null){
				TRProgram t = cgv.programs.get(key);
				if(t.isOverride()){
					System.out.println("Overriding method: "+key);
					getPrograms().put(key, t);
				} else{
					System.err.println("Function "+key+" in "+cgv.name+" should be defined as #override");
					getPrograms().put(key, t);
				}
			} else {
				getPrograms().put(key, cgv.programs.get(key));
			}
		}
		for(IMPLIES i: cgv.resolutionRules){
			beliefs.addRule(i);
		}
		for(IMPLIES i: cgv.backResolutionRules){
			BCrules.addRule(i);
		}
		for(IMPLIES i: cgv.truthManRules){
			beliefs.addContradictionRule(i);
		}
		
	}
	@Override
	public QueryResult queryAll(IFormula formula) {
		// TODO Auto-generated method stub
		return null;
	}
	@Override
	public void removeBelief(Predicate belief) {
		// TODO Auto-generated method stub
		
	}

}
