/*
 * PlatformGUI.java
 *
 * Created on 14 November 2005, 06:39
 */
package com.agentfactory.visualiser;

import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.Collections;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Observable;
import java.util.Observer;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JSeparator;
import javax.swing.JTree;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;

import com.agentfactory.platform.core.IAgent;
import com.agentfactory.platform.core.IAgentContainer;
import com.agentfactory.platform.core.IPlatformService;
import com.agentfactory.platform.core.IPlatformServiceManager;
import com.agentfactory.platform.core.IScheduler;
import com.agentfactory.platform.core.NoSuchAgentException;
import com.agentfactory.platform.impl.DefaultAgentPlatform;
import com.agentfactory.visualiser.core.IAgentInspector;
import com.agentfactory.visualiser.core.IInspectorFactory;
import com.agentfactory.visualiser.core.IStateManager;
import com.agentfactory.visualiser.core.IStateManagerFactory;
import com.agentfactory.visualiser.impl.DefaultInspectorFactory;
import com.agentfactory.visualiser.impl.MessageStore;
import com.agentfactory.visualiser.impl.SnapshotManager;

/**
 *
 * @author  Administrator
 */
public class Debugger extends javax.swing.JFrame {
	private static final long serialVersionUID = 1L;
	private static final String LOGNAME = Debugger.class.getName();
	
    private SnapshotManager manager;
    private Map<String, IAgentInspector> inspectorTable;
    private IScheduler scheduler;
    private IAgentContainer container;
    private AgentsNode agentsNode;
    private MessageStore messageStore;

    private List<IInspectorFactory> inspectorFactories;
    
    static {
        Logger.getLogger(LOGNAME).setLevel(Level.SEVERE);
    }

    public void start() {
        setSize(800, 600);
        setVisible(true);
    }

    public void registerInspectorFactory(IInspectorFactory factory) {
    	inspectorFactories.add(factory);
    }
    
    public void registerStateManagerFactory(IStateManagerFactory factory) {
    	manager.registerStateManagerFactory(factory);
    }
    
    /** Creates new form PlatformGUI */
    public Debugger(DefaultAgentPlatform platform) {
        container = platform;

        // set up the inspector table
    	inspectorFactories = new LinkedList<IInspectorFactory>();
        inspectorTable = new HashMap<String, IAgentInspector>();

        // The snapshot manager should be configured to observe events that
        // occur in the agent container.
        manager = new SnapshotManager();
        platform.addObserver(manager);

        // The platform viewer also observes events that occur in the agent
        // container to ensure the update of the agent model.
        scheduler = platform.getScheduler();
        messageStore = new MessageStore();

    	initComponents();

        setTitle("Debugger opened on: " + platform.getFullName());

        PlatformNode node = (PlatformNode) platformState.getModel().getRoot();
        node.init(platform, manager);
        
        platformState.setCellRenderer(new DefaultTreeCellRenderer() {
			private static final long serialVersionUID = 1L;
			ImageIcon[] state;

            { // Initialization block...
                state = new ImageIcon[4];
                state[0] = new ImageIcon(getClass().getResource("/com/agentfactory/visualiser/image/state_initiated.gif"));
                state[1] = new ImageIcon(getClass().getResource("/com/agentfactory/visualiser/image/state_active.gif"));
                state[2] = new ImageIcon(getClass().getResource("/com/agentfactory/visualiser/image/state_waiting.gif"));
                state[3] = new ImageIcon(getClass().getResource("/com/agentfactory/visualiser/image/state_suspended.gif"));
            }

            @Override
            public java.awt.Component getTreeCellRendererComponent(JTree tree, Object value, boolean selected, boolean expanded, boolean leaf, int row, boolean focused) {
                JLabel label = (JLabel) super.getTreeCellRendererComponent(tree, value.toString(), selected, expanded, leaf, row, focused);
                if (AgentNode.class.isInstance(value)) {
                    IAgent agent = ((AgentNode) value).agent;
                    label.setText(agent.getName());
                    if (agent.getState() < 4) {
                        label.setIcon(state[agent.getState()]);
                    }
                }

                return label;
            }
        });

        platformState.addMouseListener(new MouseListener() {

            public void mouseClicked(MouseEvent e) {
                if (e.getClickCount() == 2) {
                    TreePath treePath = platformState.getPathForLocation(e.getX(), e.getY());
                    if (treePath != null) {
                        TreeNode node = (TreeNode) treePath.getLastPathComponent();
                        if (AgentNode.class.isInstance(node)) {
                            AgentNode aNode = (AgentNode) node;
                            openInspector(aNode.agent);
                        }
                    }
                }
            }

            public void mouseReleased(MouseEvent mouseEvent) {
                if (mouseEvent.isPopupTrigger()) {
                    showPopup(mouseEvent);
                }
            }

            public void mousePressed(MouseEvent mouseEvent) {
                if (mouseEvent.isPopupTrigger()) {
                    showPopup(mouseEvent);
                }
            }

            public void mouseExited(java.awt.event.MouseEvent mouseEvent) {
            }

            public void mouseEntered(java.awt.event.MouseEvent mouseEvent) {
            }
        });
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jScrollPane1 = new javax.swing.JScrollPane();
        accessTree = new javax.swing.JTree();
        jPanel1 = new javax.swing.JPanel();
        jToolBar1 = new javax.swing.JToolBar();
        stopAll = new javax.swing.JButton();
        startAll = new javax.swing.JButton();
        jSeparator1 = new javax.swing.JToolBar.Separator();
        auml = new javax.swing.JButton();
        jSplitPane1 = new javax.swing.JSplitPane();
        desktop = new javax.swing.JDesktopPane();
        jScrollPane3 = new javax.swing.JScrollPane();
        platformState = new javax.swing.JTree();
        jMenuBar1 = new javax.swing.JMenuBar();
        jMenu1 = new javax.swing.JMenu();
        jMenuItem2 = new javax.swing.JMenuItem();
        jMenu2 = new javax.swing.JMenu();
        jMenuItem1 = new javax.swing.JMenuItem();
        jMenu3 = new javax.swing.JMenu();

        jScrollPane1.setViewportView(accessTree);

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setTitle("Agent Factory Debugger V1.0.0");

        jPanel1.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT));

        stopAll.setIcon(new javax.swing.ImageIcon(getClass().getResource("/com/agentfactory/visualiser/image/stopall.gif"))); // NOI18N
        stopAll.setFocusable(false);
        stopAll.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        stopAll.setMaximumSize(new java.awt.Dimension(37, 24));
        stopAll.setMinimumSize(new java.awt.Dimension(37, 24));
        stopAll.setPreferredSize(new java.awt.Dimension(37, 24));
        stopAll.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        stopAll.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                stopAllActionPerformed(evt);
            }
        });
        jToolBar1.add(stopAll);

        startAll.setIcon(new javax.swing.ImageIcon(getClass().getResource("/com/agentfactory/visualiser/image/startall.gif"))); // NOI18N
        startAll.setFocusable(false);
        startAll.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        startAll.setMaximumSize(new java.awt.Dimension(37, 24));
        startAll.setMinimumSize(new java.awt.Dimension(37, 24));
        startAll.setPreferredSize(new java.awt.Dimension(37, 24));
        startAll.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        startAll.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                startAllActionPerformed(evt);
            }
        });
        jToolBar1.add(startAll);
        jToolBar1.add(jSeparator1);

        auml.setIcon(new javax.swing.ImageIcon(getClass().getResource("/com/agentfactory/visualiser/image/auml.gif"))); // NOI18N
        auml.setFocusable(false);
        auml.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        auml.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        auml.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                aumlActionPerformed(evt);
            }
        });
        jToolBar1.add(auml);

        jPanel1.add(jToolBar1);

        getContentPane().add(jPanel1, java.awt.BorderLayout.NORTH);

        jSplitPane1.setRightComponent(desktop);

        platformState.setModel(new DefaultTreeModel(new PlatformNode()));
        jScrollPane3.setViewportView(platformState);

        jSplitPane1.setLeftComponent(jScrollPane3);

        getContentPane().add(jSplitPane1, java.awt.BorderLayout.CENTER);

        jMenu1.setText("File");

        jMenuItem2.setMnemonic('x');
        jMenuItem2.setText("Exit");
        jMenuItem2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMenuItem2ActionPerformed(evt);
            }
        });
        jMenu1.add(jMenuItem2);

        jMenuBar1.add(jMenu1);

        jMenu2.setMnemonic('T');
        jMenu2.setText("Tools");

        jMenuItem1.setMnemonic('S');
        jMenuItem1.setText("Message Sender");
        jMenuItem1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMenuItem1ActionPerformed(evt);
            }
        });
        jMenu2.add(jMenuItem1);

        jMenuBar1.add(jMenu2);

        jMenu3.setText("Views");
        jMenuBar1.add(jMenu3);

        setJMenuBar(jMenuBar1);

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void jMenuItem2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jMenuItem2ActionPerformed
        System.exit(0);
    }//GEN-LAST:event_jMenuItem2ActionPerformed

    private void jMenuItem1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jMenuItem1ActionPerformed
    }//GEN-LAST:event_jMenuItem1ActionPerformed

    @SuppressWarnings("unchecked")
    private void stopAllActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_stopAllActionPerformed
    	if (agentsNode != null) {
    		Enumeration e = agentsNode.children();
	        while (e.hasMoreElements()) {
	            AgentNode node = (AgentNode) e.nextElement();
	            scheduler.suspend(node.agent.getName());
	        }
	    }
	}//GEN-LAST:event_stopAllActionPerformed

	@SuppressWarnings("unchecked")
	private void startAllActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_startAllActionPerformed
	    if (agentsNode != null) {
	        Enumeration e = agentsNode.children();
	        while (e.hasMoreElements()) {
	            AgentNode node = (AgentNode) e.nextElement();
	            scheduler.resume(node.agent.getName());
	        }
	    }
	}//GEN-LAST:event_startAllActionPerformed

    private void removeActionPerformed(java.awt.event.ActionEvent evt) {
        TreePath[] values = platformState.getSelectionPaths();
        if (values != null) {
            for (TreePath path : values) {
                AgentNode node = (AgentNode) path.getLastPathComponent();
                System.out.println("Remove: " + node);
            }
        }
    }

    private void startActionPerformed(java.awt.event.ActionEvent evt) {
        TreePath[] values = platformState.getSelectionPaths();
        if (values != null) {
            for (TreePath path : values) {
                AgentNode node = (AgentNode) path.getLastPathComponent();
                scheduler.resume(node.agent.getName());
            }
        }
    }

    private void stepActionPerformed(java.awt.event.ActionEvent evt) {
        TreePath[] values = platformState.getSelectionPaths();
        if (values != null) {
            for (TreePath path : values) {
                AgentNode node = (AgentNode) path.getLastPathComponent();
                scheduler.step(node.agent.getName());
            }
        }
    }

    private void stopActionPerformed(java.awt.event.ActionEvent evt) {
        TreePath[] values = platformState.getSelectionPaths();
        if (values != null) {
            for (TreePath path : values) {
                AgentNode node = (AgentNode) path.getLastPathComponent();
                scheduler.suspend(node.agent.getName());
            }
        }
    }
    
    private AUMLView aumlView;

private void aumlActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_aumlActionPerformed
    if (aumlView == null) {
        aumlView = new AUMLView(messageStore);
        this.desktop.add(aumlView);
        messageStore.addObserver(aumlView);
    } else {
        aumlView.moveToFront();
    }
    aumlView.setVisible(true);
}//GEN-LAST:event_aumlActionPerformed
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JTree accessTree;
    private javax.swing.JButton auml;
    private javax.swing.JDesktopPane desktop;
    private javax.swing.JMenu jMenu1;
    private javax.swing.JMenu jMenu2;
    private javax.swing.JMenu jMenu3;
    private javax.swing.JMenuBar jMenuBar1;
    private javax.swing.JMenuItem jMenuItem1;
    private javax.swing.JMenuItem jMenuItem2;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane3;
    private javax.swing.JToolBar.Separator jSeparator1;
    private javax.swing.JSplitPane jSplitPane1;
    private javax.swing.JToolBar jToolBar1;
    private javax.swing.JTree platformState;
    private javax.swing.JButton startAll;
    private javax.swing.JButton stopAll;
    // End of variables declaration//GEN-END:variables
    
    private void showPopup(MouseEvent mouseEvent) {
        TreePath path = platformState.getSelectionPath();
        if ((path != null) && (AgentNode.class.isInstance(path.getLastPathComponent()))) {
            JPopupMenu menu = new JPopupMenu();
            JMenuItem item = new JMenuItem("Open Inspector");
            item.addActionListener(new ActionListener() {

                public void actionPerformed(ActionEvent e) {
                    AgentNode node = (AgentNode) platformState.getSelectionPath().getLastPathComponent();
                    openInspector(node.agent);
                }
            });
            item.setFont(item.getFont().deriveFont(Font.BOLD));
            menu.add(item);
            menu.add(new JSeparator());

            item = new JMenuItem("Start");
            item.setIcon(new ImageIcon(getClass().getResource("/com/agentfactory/visualiser/image/start.gif")));
            item.addActionListener(new ActionListener() {

                public void actionPerformed(ActionEvent e) {
                    startActionPerformed(e);
                }
            });
            menu.add(item);

            item = new JMenuItem("Stop");
            item.setIcon(new ImageIcon(getClass().getResource("/com/agentfactory/visualiser/image/stop.gif")));
            item.addActionListener(new ActionListener() {

                public void actionPerformed(ActionEvent e) {
                    stopActionPerformed(e);
                }
            });
            menu.add(item);

            item = new JMenuItem("Step");
            item.setIcon(new ImageIcon(getClass().getResource("/com/agentfactory/visualiser/image/step.gif")));
            item.addActionListener(new ActionListener() {

                public void actionPerformed(ActionEvent e) {
                    stepActionPerformed(e);
                }
            });
            menu.add(item);

            menu.add(new JSeparator());
            item = new JMenuItem("Terminate");
            item.setIcon(new ImageIcon(getClass().getResource("/com/agentfactory/visualiser/image/remove.gif")));
            item.addActionListener(new ActionListener() {

                public void actionPerformed(ActionEvent e) {
                    removeActionPerformed(e);
                }
            });
            menu.add(item);

            menu.show(this, mouseEvent.getX(), mouseEvent.getY());
        }
    }

    private IInspectorFactory getInspectorFactory(IStateManager manager) {
        for (IInspectorFactory factory: inspectorFactories) {
            if (factory.handles(manager)) return factory;
        }
        
        return new DefaultInspectorFactory();
    }
    
    private void openInspector(IAgent agent) {
        IAgentInspector inspector = (IAgentInspector) inspectorTable.get(agent.getName());

        if (inspector == null) {
            IStateManager stateManager = manager.getStateManager(agent.getName());
            inspector = getInspectorFactory(stateManager).create(stateManager, scheduler);
            inspectorTable.put(agent.getName(), inspector);
            desktop.add(inspector.getInternalFrame());
        } else {
            inspector.getInternalFrame().moveToFront();
        }
        inspector.getInternalFrame().setVisible(true);
    }

    private void updateTreeView() {
        int i = 0;
        while (i < platformState.getRowCount()) {
            platformState.expandRow(i++);
        }
    }

    public class PlatformNode implements TreeNode {

        List<TreeNode> nodes = new LinkedList<TreeNode>();
        DefaultAgentPlatform platform;

        public void init(DefaultAgentPlatform platform, SnapshotManager manager) {
            this.platform = platform;
            nodes.add(new StringNode(this, "Name: " + platform.getName()));
            nodes.add(new StringNode(this, "Domain: " + platform.getDomain()));

            agentsNode = new AgentsNode(this);
            manager.addObserver(agentsNode);
            nodes.add(agentsNode);

            nodes.add(new ServicesNode(this, platform.getPlatformServiceManager()));
        }

        public TreeNode getChildAt(int childIndex) {
            return nodes.get(childIndex);
        }

        public int getChildCount() {
            return nodes.size();
        }

        public TreeNode getParent() {
            return null;
        }

        public int getIndex(TreeNode node) {
            return nodes.indexOf(node);
        }

        public boolean getAllowsChildren() {
            return true;
        }

        public boolean isLeaf() {
            return nodes.isEmpty();
        }

        @SuppressWarnings("unchecked")
        public Enumeration children() {
            return Collections.enumeration(nodes);
        }

        @Override
        public String toString() {
            return "Platform Details";
        }
    }

    public class AgentsNode implements TreeNode, Observer {

        List<AgentNode> nodes;
        TreeNode parent;

        public AgentsNode(TreeNode parent) {
            this.parent = parent;
            nodes = new LinkedList<AgentNode>();
        }

        public int getChildCount() {
            return nodes.size();
        }

        public TreeNode getParent() {
            return parent;
        }

        public int getIndex(TreeNode node) {
            return nodes.indexOf(node);
        }

        public boolean getAllowsChildren() {
            return true;
        }

        public boolean isLeaf() {
            return nodes.isEmpty();
        }

        @SuppressWarnings("unchecked")
        public Enumeration children() {
            return Collections.enumeration(nodes);
        }

        @Override
        public String toString() {
            return "Agents";
        }

        public synchronized void update(Observable o, Object arg) {
            if (o instanceof SnapshotManager && arg instanceof String) {
                String[] info = ((String) arg).split(" ");
                if (info[0].equals("created")) {
                    try {
                        IAgent agent = container.getAgentByName(info[1]);
                        nodes.add(new AgentNode(this, agent));
                        ((DefaultTreeModel) platformState.getModel()).nodesWereInserted(this, new int[]{nodes.size() - 1});
                        if (Observable.class.isInstance(agent)) {
                            ((Observable) agent).addObserver(messageStore);
                        }
                    } catch (NoSuchAgentException ex) {
                        Logger.getLogger(Debugger.class.getName()).log(Level.SEVERE, null, ex);
                    }
                } else if (info[0].equals("terminated")) {
                    try {
                        IAgent agent = container.getAgentByName(info[1]);
                        int i = 0;
                        boolean found = false;
                        while (!found && !nodes.get(i).agent.equals(agent)) {
                            i++;
                        }
                        AgentNode node = nodes.remove(i);
                        ((DefaultTreeModel) platformState.getModel()).nodesWereRemoved(parent, new int[]{i}, new Object[]{node});
                        if (Observable.class.isInstance(agent)) {
                            ((Observable) agent).deleteObserver(messageStore);
                        }
                    } catch (NoSuchAgentException ex) {
                        Logger.getLogger(Debugger.class.getName()).log(Level.SEVERE, null, ex);
                    }
                }

                updateTreeView();
            }
        }

        public TreeNode getChildAt(int childIndex) {
            return nodes.get(childIndex);
        }
    }

    public class AgentNode implements TreeNode {

        TreeNode parent;
        IAgent agent;

        public AgentNode(TreeNode parent, IAgent agent) {
            this.parent = parent;
            this.agent = agent;
        }

        public int getChildCount() {
            return 0;
        }

        public TreeNode getParent() {
            return parent;
        }

        public int getIndex(TreeNode node) {
            return -1;
        }

        public boolean getAllowsChildren() {
            return false;
        }

        public boolean isLeaf() {
            return true;
        }

        @SuppressWarnings("unchecked")
        public Enumeration children() {
            return Collections.enumeration(Collections.emptyList());
        }

        @Override
        public String toString() {
            return agent.getName();
        }

        public TreeNode getChildAt(int childIndex) {
            return null;
        }
    }

    public class StringNode implements TreeNode {
        TreeNode parent;
        String name;

        public StringNode(TreeNode parent, String name) {
            this.parent = parent;
            this.name = name;
        }

        public int getChildCount() {
            return 0;
        }

        public TreeNode getParent() {
            return parent;
        }

        public int getIndex(TreeNode node) {
            return -1;
        }

        public boolean getAllowsChildren() {
            return false;
        }

        public boolean isLeaf() {
            return true;
        }

        @SuppressWarnings("unchecked")
        public Enumeration children() {
            return Collections.enumeration(Collections.emptyList());
        }

        @Override
        public String toString() {
            return name;
        }

        public TreeNode getChildAt(int childIndex) {
            return null;
        }
    }

    public class ServicesNode implements TreeNode, Observer {

        List<StringNode> nodes;
        TreeNode parent;

        public ServicesNode(TreeNode parent, IPlatformServiceManager manager) {
            this.parent = parent;
            nodes = new LinkedList<StringNode>();
            for (IPlatformService service : manager.getFilteredServices(IPlatformService.class)) {
                nodes.add(new StringNode(this, service.getName()));
            }
        }

        public int getChildCount() {
            return nodes.size();
        }

        public TreeNode getParent() {
            return parent;
        }

        public int getIndex(TreeNode node) {
            return nodes.indexOf(node);
        }

        public boolean getAllowsChildren() {
            return true;
        }

        public boolean isLeaf() {
            return nodes.isEmpty();
        }

        @SuppressWarnings("unchecked")
		public Enumeration children() {
            return Collections.enumeration(nodes);
        }

        @Override
        public String toString() {
            return "Platform Services";
        }

        public synchronized void update(Observable o, Object arg) {
        }

        public TreeNode getChildAt(int childIndex) {
            return nodes.get(childIndex);
        }
    }
}
